/*
 *	FGA - Fast Genetic Algorithm
 *	Copyright (C) 2007 Alessandro Presta (alessandro.presta@gmail.com)
 *	All rights reserved.
 *
 *	This library is free for commercial and non-commercial use as long as
 *	the following conditions are aheared to. The following conditions 
 *	apply to all code found in this distribution, be it the library itself
 *	or the included examples. All the documentation included with this 
 *	distribution is covered by the same copyright terms.
 *	
 *	Copyright remains Alessandro Presta's, and as such any Copyright 
 *	notices in the code are not to be removed.
 *	If this package is used in a product, Alessandro Presta should be 
 *	given attribution as the author of the parts of the library used.
 *	This can be in the form of a textual message at program startup or
 *	in documentation (online or textual) provided with the package.
 *
 *	Redistribution and use in source and binary forms, with or without
 *	modification, are permitted provided that the following conditions
 *	are met:
 *	1. Redistributions of source code must retain the copyright
 *	   notice, this list of conditions and the following disclaimer.
 *	2. Redistributions in binary form must reproduce the above copyright
 *	   notice, this list of conditions and the following disclaimer in the
 *	   documentation and/or other materials provided with the distribution.
 *	3. All advertising materials mentioning features or use of this software
 *	   must display the following acknowledgement:
 *	   "This product includes the Fast Genetic Algorithm library written by
 *	   Alessandro Presta (alessandro.presta@gmail.com)"
 *
 * 	THIS SOFTWARE IS PROVIDED BY ALESSANDRO PRESTA ``AS IS'' AND
 * 	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 *	FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * 	OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * 	HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * 	LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * 	OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * 	SUCH DAMAGE.
 * 
 * 	The licence and distribution terms for any publically available version or
 * 	derivative of this code cannot be changed.  i.e. this code cannot simply be
 * 	copied and put under another distribution licence
 *	(including the GNU Public Licence).
 */

#include <cstdlib>
#include <cstring>
#include <ctime>
#include <math.h>
#include <algorithm>
#include <fstream>
#include <iostream>

using namespace std;

#ifndef WIN32
#define CLEAR "clear"
#else
#define CLEAR "cls"
#endif 
#define PRINT_ALL() { \
						system(CLEAR); \
						cout << "Brute force algorithm" << endl << endl; \
						if (L <= 12) \
							cout << "Progress:                " << (int)pct << "% (" << count << " of " << L_fact << " paths generated)" << endl << endl; \
						else \
							cout << "Progress:                " << count << " paths generated" << endl << endl; \
						cout << "Best cost:               " << best_cost << endl << endl; \
						cout << "Best path:" << endl; \
						for (int i = 0; i < L; ++i) \
							cout << best_path[i] << " "; \
						cout << endl << endl; \
		    		}

int L; // number of nodes in the graph
int MAX_COST; // maximum cost for a road
int **cost; // cost for the road between nodes [i][j]
int *best_path;
int best_cost;

long fact(int n)
{
	if (n == 0)
		return 1;
	return n * fact(n - 1);	
}

int main()
{
	srand(time(NULL));
	ifstream in_file("graph.txt");
	in_file >> L;
	cost = new int*[L];
	for (int i = 0; i < L; ++i) {
		cost[i] = new int[L];
		for (int j = 0; j < L; ++j)
			cost[i][j] = 0;
	}
	best_path = new int[L];
	MAX_COST = 0;
	for (int i = 0; i < L - 1; ++i)
		for (int j = i + 1; j < L; ++j) {
			in_file >> cost[i][j];
			cost[j][i] = cost[i][j];
			MAX_COST = max(cost[i][j], MAX_COST);
		}
	in_file.close();
	best_cost = MAX_COST * L;
	int *path = new int[L];
	for (int i = 0; i < L; ++i)
		path[i] = i;
	int count = 0;
	long L_fact = fact(L);
	float pct = 0;
	do {
		int total_cost = 0;
		for (int i = 0; i < L - 1; ++i)
			total_cost += cost[path[i]][path[i + 1]];
		total_cost += cost[path[L - 1]][path[0]];
		if (total_cost < best_cost) {
			best_cost = total_cost;
			memcpy(best_path, path, L * sizeof(int));
			PRINT_ALL();
		}
		count++;
		if (count % 10000 == 0)
			PRINT_ALL();
		if (L <= 20) {
			pct = (float)count / L_fact * 100;
			if (pct - (int)pct < 0.5)
				pct = floor(pct);
			else
				pct = ceil(pct);
		}
	} while (next_permutation(path, path + L));
	PRINT_ALL();
	for (int i = 0; i < L; ++i)
		delete[] cost[i];
	delete[] cost;
	delete[] best_path;
	delete[] path;
	return 0;
}
